clear;close all;clc;
tic
rand('state',sum(100*clock));   %reset the random number generator

N=200;     % Number of points in each round-trip time                   
Nrt=12000; % The number of cycles in the OEO               
Nmax=10000; % The number of iterations to reach a steady state is (Nrt-Nmax)=2000                           
T=0.28e-6;%1.5*56/3e8; %round trip time (T is like tau=L/c in the paper )                 
R=50;
RFampCorr=1; %Correction factor to RF amplifier gain
P0=30e-3; %Input optical power
VpiAC=3.14;  %V_pi,AC                      
VpiDC=3.14; %V_pi_DC=3.14 V
Vb=3.14; %Bias voltage Vb              
eta=1;  %The parameter etha as in the paper                        
rhoDC=0.8; %responsivity of the photodetector at 10 GHz (A/W)           
rho=0.8;  %responsivity of the photodetector at DC (A/W)     
Ga=7.5;   %amplifier voltage gain             
attenRF=10^(0); %RF attenuation (default value = 1)
FilterLoss=10^(0); %Filter loss
Gwidth=20e6;  %The bandwidth of the filter (like Gamma in the paper)              
CouplerLoss=10^(0);
rho_N=10^(-20);
Gs=1.5;
alpha=double((2*VpiDC*Gs)/(eta*pi*P0*rho*R*Ga)); %insertion loss                      

phase_noise_sum=zeros(1,N*Nmax);
phase_noise_frq_sum=zeros(1,N*Nmax);
N_avg=350;

display(['Generating figure 4. Please wait.']);

for ii=1:N_avg

[phase_noise,phase_noise_frq,v0_v,t_v,freq_v,phase_noise_RF,phi_v]=SingleOEO_main(N,Nrt,Nmax,T,alpha,RFampCorr,P0,VpiAC,VpiDC,Vb,eta,rhoDC,rho,Ga,...
    attenRF,FilterLoss,Gwidth,CouplerLoss,rho_N);

phase_noise_sum=phase_noise_sum+phase_noise;
phase_noise_frq_sum=phase_noise_frq_sum+phase_noise_frq;

display(['Iteration ',num2str(ii),' of ',num2str(N_avg)]);
end;

phase_noise_avg=phase_noise_sum./N_avg;
phase_noise_frq_avg=phase_noise_frq_sum./N_avg;

delta=rho_N*(Ga^2)/P0;
Vosc=sqrt(2*R*P0);
G_Vosc=sqrt(1-delta/T); %based on equation 23 in Yao-Maleki paper ,(Gs*(2*VpiDC)/(pi*Vosc))*besselj(1,pi*Vosc/VpiDC);
G_v=double(Gwidth/2*i./(freq_v+i*Gwidth/2));
F=abs(G_v);
%based on equation 21 in Yao-Maleki paper
phase_Maleki=10*log10(delta./(1+abs(F.*G_Vosc).^2-2.*F.*abs(G_Vosc).*cos(2.*pi.*freq_v.*T)));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%figure 4 in the paper
sim_end=toc;

f_start=find(freq_v>=10^2,1);
f_end=find(freq_v>=10^7,1);
f_end_fit=find(freq_v>=10^6,1);
freq_axis=freq_v(f_start:f_end);
phase_noise_axis=phase_noise_avg(f_start:f_end);
phase_Maleki_axis=phase_Maleki(f_start:f_end);

freq_axis_fit=freq_v(f_start:f_end_fit);
y_fit=-55.5-20.*log10(freq_axis_fit);

figure(1);
semilogx(freq_axis,phase_noise_axis,'-b','LineWidth',0.5);
hold on;
semilogx(freq_axis_fit,y_fit,'-.k','LineWidth',1.5);
semilogx(freq_axis,phase_Maleki_axis,'--r','LineWidth',1.5);
hold off;
box off;
xlabel('Frequency offset (Hz)','fontname','Times New Roman','fontsize',12);
ylabel('Phase noise (dBc/Hz)','fontname','Times New Roman','fontsize',12);
set(gca,'fontname','Times New Roman','fontsize',12);
set(gca,'Ylim',[-180 -100],'Ytick',[-180,-160,-140,-120,-100],'YtickLabel',{'-180';'-160';'-140';'-120';'-100'});



