clear;close all;clc;
tic
rand('state',sum(100*clock));   %reset the random number generator

N=200;     % Number of points in each round-trip time                   
Nrt=12000; % The number of cycles in the OEO               
Nmax=10000; % The number of iterations to reach a steady state is (Nrt-Nmax)=2000                           
T=1.5*56/3e8; %round trip time (T is like tau=L/c in the paper )                 
alpha=10^(-0.477); %insertion loss                      
RFampCorr=1; %Correction factor to RF amplifier gain
P0=30e-3; %Input optical power
VpiAC=3.14;  %V_pi,AC                      
VpiDC=3.14; %V_pi_DC=3.14 V
Vb=3.14; %Bias voltage Vb              
eta=1;  %The parameter etha as in the paper                        
rhoDC=0.8; %responsivity of the photodetector at 10 GHz (A/W)           
rho=0.8;  %responsivity of the photodetector at DC (A/W)     
Ga=7.5;   %amplifier voltage gain             
attenRF=10^(0); %RF attenuation (default value = 1)
FilterLoss=10^(0); %Filter loss
Gwidth=20e6;  %The bandwidth of the filter (like Gamma in the paper)              
CouplerLoss=10^(0);
rho_N=10^(-20);

N_v=N*Nmax;
R=50;
DT=T/N;
T_v=N_v*DT;
DF_v=1/T_v;

phase_noise_sum=zeros(1,N*Nmax);
phase_noise_frq_sum=zeros(1,N*Nmax);
ampl_noise_sum=zeros(1,N*Nmax);
N_avg=350;%350;

display(['Generating figure 5. Please wait.']);

for ii=1:N_avg
[phase_noise,phase_noise_frq,v0_v,t_v,freq_v,phase_noise_RF,phi_v]=SingleOEO_main(N,Nrt,Nmax,T,alpha,RFampCorr,P0,VpiAC,VpiDC,Vb,eta,rhoDC,rho,Ga,...
    attenRF,FilterLoss,Gwidth,CouplerLoss,rho_N);

phase_noise_sum=phase_noise_sum+phase_noise;
phase_noise_frq_sum=phase_noise_frq_sum+phase_noise_frq;

%Amplitude noise
V0_v=double(fftshift(fft(v0_v))/N_v);
Pw=0.5*abs(V0_v).^2/R;                  % Power as a function of frequency [W]
Pavg=sum(Pw);   
abs_V0_v=double(fftshift(fft(abs(v0_v)))/N_v);
ampl_noise_frq=10*log10(abs(abs_V0_v).^2/R/2/Pavg/DF_v); %Amplitude noise
ampl_noise_sum=ampl_noise_sum+ampl_noise_frq;
%end amplitude noise

display(['Iteration ',num2str(ii),' of ',num2str(N_avg)]);
end;

phase_noise_avg=phase_noise_sum./N_avg;
phase_noise_frq_avg=phase_noise_frq_sum./N_avg;
ampl_noise_avg=ampl_noise_sum./N_avg;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%figure 5 in the paper
sim_end=toc;

f_start=find(freq_v>=0,1);
f_end=find(freq_v>=10^7,1);
freq_axis=freq_v(f_start:f_end);
phase_noise_axis=phase_noise_avg(f_start:f_end);
phase_noise_frq_avg_axis=phase_noise_frq_avg(f_start:f_end);
ampl_noise_frq_axis=ampl_noise_avg(f_start:f_end);


close all;
figure(1);
semilogx(freq_axis,phase_noise_axis,'-','LineWidth',0.5,'Color',[170,170,170]./255);
hold on;
semilogx(freq_axis,phase_noise_frq_avg_axis,'-k','LineWidth',0.5);
hold off;
hold on;
semilogx(freq_axis,ampl_noise_frq_axis,'LineWidth',0.5,'Color',[230,230,230]./255);
hold off;
box off;
xlabel('Frequency offset (Hz)','fontname','Times New Roman','fontsize',12);
ylabel('Phase noise (dBc/Hz)','fontname','Times New Roman','fontsize',12);
set(gca,'fontname','Times New Roman','fontsize',12);
set(gca,'Ylim',[-180 -100],'Ytick',[-180,-160,-140,-120,-100],'YtickLabel',{'-180';'-160';'-140';'-120';'-100'});



